/*  styling for the datagridfield */

div.ArchetypesDataGridWidget table td img {
    display: block;
    cursor: pointer;
}

div.ArchetypesDataGridWidget table td input, 
div.ArchetypesDataGridWidget table td select {
    width:100%;
}
